﻿Imports Microsoft.Kinect
Imports System.ComponentModel

Public Class MainWindow
    Private myKinect As KinectSensor
    Private myBodyFrameReader As BodyFrameReader
    Private myBodies As Body()

    Private myHandPositionX As Double
    Private myHandPositionY As Double
    Private myColorSpacePoint As ColorSpacePoint
    Private myBytesPerPixel As Integer = 4 'フルカラーの場合は32ビットで1バイトは8ビットなので4バイトとなる

    Private myColorFrameReader As ColorFrameReader = Nothing
    Private colorBitmap As WriteableBitmap = Nothing
    Private ColorImagePixelData As Byte()
    Private BytesPerPixel As Integer = 4

    Private commonJoint As Joint
    Private point As DepthSpacePoint
    Private Index As Integer = 0
    Private no As Integer = 0

    Private Sub MainWindow_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded
        myKinect = KinectSensor.GetDefault
        If myKinect Is Nothing = False Then

            Dim myDepthDescription = myKinect.DepthFrameSource.FrameDescription
            myKinect.Open()
            myBodyFrameReader = myKinect.BodyFrameSource.OpenReader
            AddHandler myBodyFrameReader.FrameArrived, AddressOf myBodyFrameReader_FrameArrived
            myBodies = New Body(myKinect.BodyFrameSource.BodyCount - 1) {}
            myColorFrameReader = myKinect.ColorFrameSource.OpenReader
            Dim myColorFrameDescripton As FrameDescription = myKinect.ColorFrameSource.CreateFrameDescription(ColorImageFormat.Bgra)
            ColorImagePixelData = New Byte(myColorFrameDescripton.Width * myColorFrameDescripton.Height * BytesPerPixel - 1) {}
            colorBitmap = New WriteableBitmap(myColorFrameDescripton.Width, myColorFrameDescripton.Height, 96.0, 96.0, PixelFormats.Bgr32, Nothing)
            AddHandler myColorFrameReader.FrameArrived, AddressOf myColorFrameReader_FrameArrived
        End If
    End Sub

    Private Sub myBodyFrameReader_FrameArrived(sender As Object, e As BodyFrameArrivedEventArgs)
        'CanvasBody.Children.Clear()
        Using myBodyFrame = e.FrameReference.AcquireFrame
            If myBodyFrame Is Nothing = True Then
                Return
            End If
            myBodyFrame.GetAndRefreshBodyData(myBodies)
        End Using
        CanvasBody.Children.Clear()
        For Each body In myBodies

            For Each joint In body.Joints
                If joint.Value.TrackingState = TrackingState.Tracked Then
                    DrawEllipse(joint.Value, 10, Brushes.Transparent)
                    If joint.Value.JointType = JointType.HandRight Then
                        If body.Joints(JointType.HandTipRight).TrackingState = TrackingState.Tracked Then
                            myColorSpacePoint = myKinect.CoordinateMapper.MapCameraPointToColorSpace(body.Joints(JointType.HandTipRight).Position)
                            myHandPositionX = CInt(myColorSpacePoint.X * 0.5) 'CInt((body.Joints(JointType.ThumbRight).Position.X))
                            myHandPositionY = CInt(myColorSpacePoint.Y * 0.5) 'CInt((body.Joints(JointType.ThumbRight).Position.Y))
                            DrawHandState(body.Joints(JointType.HandRight), body.HandRightState, body.HandRightConfidence)

                        End If

                    End If
                    'ElseIf joint.Value.TrackingState = TrackingState.Inferred Then
                    '    DrawEllipse(joint.Value, 10, Brushes.Transparent)
                End If
            Next
        Next
    End Sub

    Private Sub myColorFrameReader_FrameArrived(sender As Object, e As ColorFrameArrivedEventArgs)

        Dim colorFramePorcessd As Boolean = False
        Using myColorFrame As ColorFrame = e.FrameReference.AcquireFrame
            If myColorFrame Is Nothing = False Then
                Dim myColorFrameDescription As FrameDescription = myColorFrame.FrameDescription
                If myColorFrameDescription.Width = colorBitmap.PixelWidth AndAlso myColorFrameDescription.Height = colorBitmap.PixelHeight Then
                    If myColorFrame.RawColorImageFormat = ColorImageFormat.Bgra Then
                        myColorFrame.CopyRawFrameDataToArray(ColorImagePixelData)
                    Else
                        myColorFrame.CopyConvertedFrameDataToArray(ColorImagePixelData, ColorImageFormat.Bgra)
                    End If
                    colorFramePorcessd = True
                End If
            End If
        End Using
        If colorFramePorcessd = True Then
            colorBitmap.WritePixels(New Int32Rect(0, 0, colorBitmap.PixelWidth, colorBitmap.PixelHeight), ColorImagePixelData, colorBitmap.PixelWidth * BytesPerPixel, 0)
            roomImage.Source = colorBitmap
        End If
    End Sub
    Private Sub DrawHandState(joint As Joint, state As HandState, myTrackingConfidence As TrackingConfidence)
        If myTrackingConfidence <> TrackingConfidence.High Then
            Return
        End If

        If state = HandState.Open Then
            Index = 0
        ElseIf state = HandState.Closed Then

            If point.X > Image1.GetValue(Canvas.LeftProperty) And point.X < Image1.GetValue(Canvas.LeftProperty) + Image1.Width AndAlso point.Y > Image1.GetValue(Canvas.TopProperty) And point.Y < Image1.GetValue(Canvas.TopProperty) + Image1.Height Then
                Index = 1
            End If
            If point.X > Image2.GetValue(Canvas.LeftProperty) And point.X < Image2.GetValue(Canvas.LeftProperty) + Image2.Width AndAlso point.Y > Image2.GetValue(Canvas.TopProperty) And point.Y < Image2.GetValue(Canvas.TopProperty) + Image2.Height Then
                Index = 2
            End If
        End If
    End Sub

    Private Sub MainWindow_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        If myKinect Is Nothing = False Then
            myKinect.Close()
            myKinect = Nothing
        End If
    End Sub

    Private Sub DrawEllipse(myJoint As Joint, R As Integer, myBrush As Brush)

        Dim myEllipse = New Ellipse
        With myEllipse
            .Width = R
            .Height = R
            .Fill = myBrush
        End With

        point = myKinect.CoordinateMapper.MapCameraPointToDepthSpace(myJoint.Position)
        If point.X < 0 OrElse point.Y < 0 Then
            Return
        End If
        Canvas.SetLeft(myEllipse, point.X - (R / 2))
        Canvas.SetTop(myEllipse, point.Y - (R / 2))
        If myJoint.JointType = JointType.HandRight Then
            'TextBlock1.Text = Index
            Select Case Index
                Case 1
                    MediaElement1.Play()
                Case 2
                    MediaElement2.Play()

            End Select
        End If
        CanvasBody.Children.Add(myEllipse)
    End Sub

    Private Sub MediaElement1_MediaEnded(sender As Object, e As RoutedEventArgs) Handles MediaElement1.MediaEnded
        MediaElement1.Close()
    End Sub

    Private Sub MediaElement2_MediaEnded(sender As Object, e As RoutedEventArgs) Handles MediaElement2.MediaEnded
        MediaElement2.Close()
    End Sub
End Class
